<?php
	/**
	 * Elgg CAS authentication
	 * 
	 * @package cas_auth
	 * @license http://www.gnu.org/licenses/gpl.html
	 * @author Xavier Roussel <xavier.roussel@uvsq.fr>  (Modified for UNL CAS by Eric Rasmussen)
	 * @copyright UVSQ 2008
	 * @link http://www.uvsq.fr
	 */

	require_once 'UNL/Services/Peoplefinder.php';
	
	//In order to avoid a "__PHP_Incomplete_Class" problem when we unserialize the Peoplefinder data we need to set up the class here
	//http://us2.php.net/manual/en/function.unserialize.php
	class UNL_Peoplefinder_Record {
	 public $cn;
	 public $ou;	 
	 public $eduPersonNickname;
	 public $eduPersonPrimaryAffiliation;
	 public $givenName;
	 public $displayName;
	 public $mail;
	 public $postalAddress;
	 public $sn;
	 public $telephoneNumber;
	 public $title;
	 public $uid;
	 public $unlHRPrimaryDepartment;
	 public $unlHRAddress;
	 public $unlSISClassLevel;
	 public $unlSISCollege;
	 public $unlSISLocalAddr1;
	 public $unlSISLocalAddr2;
	 public $unlSISLocalCity;
	 public $unlSISLocalPhone;
	 public $unlSISLocalState;
	 public $unlSISLocalZip;
	 public $unlSISPermAddr1;
	 public $unlSISPermAddr2;
	 public $unlSISPermCity;
	 public $unlSISPermState;
	 public $unlSISPermZip;
	 public $unlSISMajor;
	 public $unlEmailAlias;
	 
	/**
     * Takes in a string from the LDAP directory, usually formatted like:
     *     ### ___ UNL 68588-####
     *    Where ### is the room number, ___ = Building Abbreviation, #### zip extension
     *
     * @param string
     * @return array Associative array.
     */
     function formatPostalAddress() {
        /* this is a faculty postal address
            Currently of the form:
            ### ___ UNL 68588-####
            Where ### is the room number, ___ = Building Abbreviation, #### zip extension
        */
        /**
         * We assumed that the address format is: ### ___ UNL 68588-####.
         * Some 'fortunate' people have addresses not in this format.
         */
        //RLIM
        // treat UNL as the delimiter for the streetaddress and zip
        if (strpos($this->postalAddress,'UNL')) {
            $addressComponent = explode('UNL', $this->postalAddress);
        } elseif (strpos($this->postalAddress,'UNO')) {
            $addressComponent = explode('UNO', $this->postalAddress);
        } elseif (strpos($this->postalAddress,'Omaha')) {
            $addressComponent = explode('Omaha', $this->postalAddress);
        } else {
            $addressComponent = array($this->postalAddress);
        }
        $address['region']         = 'NE';
        $address['street-address'] = trim($addressComponent[0]);
        if (isset($addressComponent[1])) {
            $address['postal-code'] = trim($addressComponent[1]);
        } else {
            $address['postal-code'] = '';
        }
        switch (substr($address['postal-code'],0,3)) {
            case '681':
                $address['locality'] = 'Omaha';
            break;
            case '685':
            default:
                $address['locality'] = 'Lincoln';
            break;
        }
        
        return $address;
     }
	}
			

	// Include main cas lib
	include_once 'cas/CAS.php'; 

	/**
	 * CAS Authentication init
	 * 
	 */
	function cas_auth_init() { 
		global $CONFIG;
		// plugin config
		$config = find_plugin_settings('cas_auth');
		// todo : send message to user
		if (!$config)
			return false;
		
		if ($_GET['loginwith'] == 'UNLlogin') { 
			if (checkCas()) {
				$_SESSION['loggedWithCAS'] = true;			
				
				$cas_user = getUserCas();
				if (casAuthenticate($cas_user)) {
					system_message(elgg_echo('loginok'));
					$cas_user = str_replace('-','_',$cas_user);
					
					//user is logged in now, this is the last step - forward based on whether they have logged in before 
            		if (!$_SESSION['user']->last_login)
            			forward('mod/profile/edit.php?firstlogin=yes');
            		else
            			forward("pg/profile/unl_" . $cas_user);
				} else {
					register_error(elgg_echo('loginerror'));
				}
			} else {
				createCas();
			}
		}
		
		// The CAS ticket is lost, log out
		if ($_SESSION['loggedWithCAS'] && !checkCas()) {
			$_SESSION['loggedWithCAS'] = false;
			forward($CONFIG->url.'/action/logout');
		} 
	}
	

	global $CONFIG;
	register_action("cas_auth/getemail",true,$CONFIG->pluginspath . "cas_auth/views/default/actions/getemail.php");
	// Register the initialisation function
	register_elgg_event_handler('init','system','cas_auth_init');
	// Register CAS logout to main logout only if user logged with CAS
	if (isset($_SESSION['loggedWithCAS']) && $_SESSION['loggedWithCAS']==true) {
		register_elgg_event_handler('logout', 'user', 'logoutCas');
	}	
	
	/* set up login page */
	register_page_handler('login', 'login_page_handler');
	
	function login_page_handler($page) {
		// If we're not logged in, display the login page
		if (!isloggedin()) {
			page_draw(elgg_echo('login'), elgg_view("account/forms/login"));
		// Otherwise, forward to the index page
		} else {
			forward();
		}
	}
	
	/**
	 * CAS client initialization
	 * 
	 */
	function initCas() {
		if (!$GLOBALS[casInitialized]) {
			$config = find_plugin_settings('cas_auth');
			phpCAS::client(CAS_VERSION_2_0, $config->casurl, (int) $config->casport , $config->casuri );
			$GLOBALS[casInitialized] = true;
		}	
		return true;
	}

	/**
	 * Force authentication
	 * 
	 */
	function createCas() {
		initCas();
		phpCAS::forceAuthentication();
		return true;
	}

	/**
	 * Check auth
	 * 
	 * @return boolean
	 */
	function checkCas() {		
		initCas();
		if (phpCAS::checkAuthentication()) {
			return true;
		}
		else 
			return false;
	}

	/**
	 * Get CAS user
	 * 
	 * @return string name of the user
	 */
	function getUserCas() {
		return phpCAS::getUser();
	}

	/**
	 * CAS logout
	 * 
	 */
	function logoutCas() {
		global $CONFIG;
		initCas();
		phpCAS::logout($CONFIG->url.'/action/logout');	
		return true;
	}

   /**
    * Perform an CAS authentication check
    * 
    * @param string $username
    * @return boolean
    */
	function casAuthenticate($username){
		if (empty($username))
			return false;

		// we're making this copy for use in the peoplefinderservices call later
		// we dont want to call peoplefinderservices here since we dont need to every time a SSO user logs in
		$casusername = $username;
		
		//We're going to make every UNL SSO user have an elgg profile name as such: unl_erasmussen2
		//and not allow friends of unl who register via elgg to pick names that begin with "unl_"
		//This way, we won't have to deal with the case where someone registers erasmussen2 on elgg, then
		//the real erasmussen2 signs in for the first time with UNL SSO and is logged in as the elgg user erasmussen2
		//rather then having a new account created.									
		$username = 'unl_' . $username;
		//Replace the hyphen in a student's name with an underscore
		$username = str_replace('-','_',$username);

		if ($user = get_user_by_username($username)) {
			// User exists, login            	        
			return login($user);
		} else {
			// Valid login but user doesn't exist
			$pf_user_info = peoplefinderServices($casusername);

			$name  = $pf_user_info->cn;

 			if (isset($_REQUEST['email'])) {
				$email = $_REQUEST['email'];
			} else {	
				if($pf_user_info->mail)
					forward($CONFIG->url . 'mod/cas_auth/views/default/account/getemail.php?e=' . $pf_user_info->mail);
				else
					forward($CONFIG->url . 'mod/cas_auth/views/default/account/getemail.php');
			}
				
			try {
				if ($user_guid = register_user($username, generate_random_cleartext_password(), $name, $email, false, 0, '', true)) {
					$thisuser = get_user($user_guid);
					
					//pre-populate profile fields with data from Peoplefinder Services
					$address = $pf_user_info->formatPostalAddress();
					$thisuser->profile_country = 'USA';
					$thisuser->profile_state = $address['region'];
					$thisuser->profile_city = $address['locality'];
					if($address['locality'] == 'Omaha') {
						$thisuser->longitude = -95.9;
						$thisuser->latitude = 41.25;
					} else { //this is going to cover Lincoln and everyone else
						$thisuser->longitude = -96.7;
						$thisuser->latitude = 40.82;
					}
						 
					return login($thisuser);
				} else {
					register_error(elgg_echo("registerbad"));
				}
			} catch (RegistrationException $r) {
				register_error($r->getMessage());
			}
 
		}
 
 	}
 
	/**
	 * Gets a UNL SSO user's info from Peoplefinder Services
	 * 
	 * @param string $username
	 * @return array of information from PF Services
	 */
	function peoplefinderServices($username){ 
		$pfrecord = unserialize(file_get_contents('http://peoplefinder.unl.edu/service.php?uid=' . $username . '&format=php'));
		return $pfrecord;
	}
?>